--!parent
--!encoding=utf-8

--****************************************************************************
-- @file      LSW CORE LIB: SCITE
-- @author    Valentin Schmidt
-- @version   0.1
--****************************************************************************

property ancestor

property _stopOnError
property _quitCommand

property _sciteHwnd
property _dirHwnd

property _listenCallback
property _listenTarget

property _timer

----------------------------------------
-- @constructor
----------------------------------------
on new (me)
	$ = me.$
	
	-- win only
	if $.OS="mac" then return FALSE
		
  -- libs
  $.import("detacher")
  $.import("msg")
  $.import("string")
  $.import("timer")
  
  -- find SciTE HWND
  me._sciteHwnd = me._findSciTE()
  
  -- find HWND of current director process
  me._dirHwnd = $.msg.stage_handle()
  
  -- check occasionally if SciTE is still running, otherwise exit
  me._timer = $.timer.create(2000, #_checkScite, me)
    
  return me
end

----------------------------------------
-- Starts input/output to SciTE output pane
-- @param {bool} [stopOnError=false]
-- @param {string} [quitCommand]
----------------------------------------
on consoleStart (me, stopOnError, quitCommand)
	$ = me.$
	me._stopOnError = stopOnError
  me._quitCommand = quitCommand
  
	-- load io handler movie script
	$.include(me.libdir&"io.ls")
	
	-- setup alertHook
	_player.alertHook = me
	
	-- create an asynchronous console
  consoleCreateAsync(#_consoleCallback, me, quitCommand)
  
  -- show welcome message
  stdOut("> Welcome to Director "&_player.productVersion&" / LSW "&$.VERSION&$.LF)
  if not voidP(quitCommand) then
  	stdOut("> Enter '"&quitCommand&"' to end" &$.LF, 1)
  end if
  stdOut($.LF)
    
end

----------------------------------------
-- Stops input/output to SciTE output pane
----------------------------------------
on consoleStop (me)
	consoleClose()
end

----------------------------------------
-- sends command to scite
----------------------------------------
on cmd (me, cmdStr)
	$ = me.$
	cmdStr = $.string.replace("\", "\\", cmdStr)
	$.msg.send_data(me._sciteHwnd, cmdStr)
end

----------------------------------------
-- start listening to WM_COPYDATA messages
----------------------------------------
on listen (me, cbHandler, cbTarget)
	$ = me.$
	
	if voidP(cbTarget) then cbTarget = _movie
	me._listenCallback = cbHandler
	me._listenTarget = cbTarget

	$.msg.listen_set_callback(#_listenCallback, me)	
	$.msg.listen_set_msg([$.msg.WM_COPYDATA])
	$.msg.listen_start()
	
end
  
----------------------------------------
--
----------------------------------------
on _findSciTE (me)
	$ = me.$
	SDI = $.msg.register_msg("SciTEDirectorInterface")
	hwndDesktop = $.msg.get_desktop_win()
	hwnd = $.msg.get_win(hwndDesktop, $.msg.GW_CHILD)
	repeat while (hwnd)
		res = $.msg.send_msg_timeout(hwnd, SDI, 0, 0, $.msg.SMTO_NORMAL, 1000, TRUE)
		if res=SDI then return hwnd
		hwnd = $.msg.get_win(hwnd, $.msg.GW_HWNDNEXT)
	end repeat
end


----------------------------------------
-- @callback
----------------------------------------
on alertHook (me, err, msg, type)
  case (type) of
		#script:

			-- show error in output pane
			err(err&": "&msg)

			-- exit with errorCode 1
			if me._stopOnError then exitCode(1)

			-- otherwise continue to run
			return -1

    otherwise:
    	return 0
    	
  end case
end

----------------------------------------
-- @callback
----------------------------------------
on _consoleCallback (me, str)
  me.$.detacher.detach(#_consoleCallbackDetached, me, str)
end

----------------------------------------
-- detached callback
----------------------------------------
on _consoleCallbackDetached (me, str)
	if not voidP(me._quitCommand) AND str=me._quitCommand then
		stdOut(">goodbye!"&me.$.LF, 1)
		exitCode(1)
	else if str starts ">" then
		delete char 1 of str
		put value(str)
	else
		do(str)
	end if
end

----------------------------------------
-- @callback
----------------------------------------
on _listenCallback (me, hwnd, msg, wParam, lParam, data)
	$ = me.$

	if ilk(data)=#bytearray then data = data.readRawString(data.length)
		
	-- for non-director mode: remove :hwnd:
	delete char 1 to (2+string(me._dirHwnd).length) of data
	
	-- replace double \ in pathes
	data = $.string.replace("\\", "\", data)
	
	-- detach callback from undebuggable xtra callback thread
  $.detacher.detach(me._listenCallback, me._listenTarget, data)
end

----------------------------------------
-- @callback
----------------------------------------
on _checkScite (me)
	if (me.$.msg.get_win_class(me._sciteHwnd)="") then
		consoleClose()
		_movie.halt()
	end if
end

--  The actions understood by SciTE are:
--  askfilename:  Return the name of the file being edited.
--  askproperty:<key>   Return the value of a property.
--  close:  Close the current file.
--  closing:  Director is closing - SciTE closes if it was started by the director.
--  currentmacro:<string>   Set the current macro to name.
--  cwd:  Change the working directory.
--  enumproperties:dyn|local|user|base|embed  Enumerate all the properties in the argument set.
--  exportashtml:<path>   Save the document in HTML format as the indicated file.
--  exportasrtf:<path>  Save the document in RTF format as the indicated file.
--  exportaspdf:<path>  Save the document in PDF format as the indicated file.
--  exportaslatex:<path>  Save the document in LaTeX format as the indicated file.
--  exportasxml:<path>  Save the document in XML format as the indicated file.
--  extender:<command>  Call the extension interface with the given command.
--  find:<string>   Search for a string, select and show it.
--  goto:<lineNumber>[,<columnNumber>]  Move caret to a particular line and make it visible. If there is a column number then select the word at that column number or move the caret there if no word is present.
--  identity:<hwndDirector>   Sets the director window handle to which SciTE sends messages. The argument is in decimal.
--  insert:<value>  Display the value in the editor pane replacing the selection.
--  loadsession:<path>  Load a session as given by the indicated file.
--  macrocommand:<command>  Execute a macro command. See the SciTE source code for the syntax of the command argument.
--  macroenable:<enable>  If enable, display menu commands in SciTE for recording and playing macros.
--  macrolist:<list>  Display a list for the user to choose from.
--  menucommand:<cmd>   Execute a menu command based on numeric ID.
--  open:<path>   Open the indicated file.
--  output:<value>  Display the value in the output pane replacing the selection.
--  property:<key>=<value>  Set a property to a value.
--  quit:   Shut down SciTE.
--  reloadproperties:   Reload properties from files.
--  replaceall:<search>\000<replace>  Replace all instances of the search string in the document with the replace string.
--  saveas:<path>   Save the document as the indicated file.
--  savesession:<path>  Save a session as given by the indicated file.
