require 'winapi'

----------------------------------------
--
----------------------------------------
--local function doubleClickHandler()
--  if editor.Focus then return end
--
--  local code = editor:GetText()
--  local pos = output.CurrentPos
--  local p1 = output:WordStartPosition(pos,true)
--  local p2 = output:WordEndPosition(pos,true)
--  if p1==p2 then return end
--
--  local search = "on "..output:textrange(p1,p2)
--
--  if string.sub(code, 1, string.len(search))==search then
--    pos = 0
--  else
--    pos = string.find(code, "[\n\r]"..search)
--  end
--  if pos then
--
--    -- goto line
--    local lineNum = editor:LineFromPosition(pos)
--    editor:GotoLine(lineNum)
--
--    -- select line
--    posEnd = editor.LineEndPosition[lineNum]
--    editor:SetSel(pos, posEnd) -- Select a range of text.
--    editor.FirstVisibleLine = lineNum - 3
--  end
--end

----------------------------------------
--
----------------------------------------
local function at (s,i)
  return s:sub(i,i)
end

----------------------------------------
--
----------------------------------------
local function splitLast (s,ch)
  local i = #s
  while i > 0 do
    if at(s,i) == ch then
      return s:sub(i+1)
    end
    i = i - 1
  end
end

----------------------------------------
--
----------------------------------------
function stripTrailSpaces()
  local sfound, se = -1, 0
  sfound, se=editor:findtext("[\t ]+$", SCFIND_REGEXP, sfound+1 )
  while sfound do
    editor:remove(sfound, se)
    sfound, se=editor:findtext("[\t ]+$", SCFIND_REGEXP, sfound+1 )
  end
end

----------------------------------------
-- replace one tab tab followed by one or more (space or tab)
-- but obey tabstops (preserves alignment)
----------------------------------------
function tabsToSpaces()
  for m in editor:match("[\\t][\\t ]*", SCFIND_REGEXP) do
    local posColumn = ( scite.SendEditor(SCI_GETCOLUMN, (m.pos ) ) )
    local poslenColumn = ( scite.SendEditor(SCI_GETCOLUMN, (m.pos + m.len) ) )
    m:replace(string.rep(' ', poslenColumn - posColumn ))
  end
end
	
----------------------------------------
-- fix EOL according to current editor's setting/detection
----------------------------------------
function fixEol()
		
	-- Note that \r and \n are never matched because in Scintilla, regular expression 
	-- searches are made line per line (stripped of end-of-line chars). 
	
	if editor.EOLMode==0 then -- CRLF mode
		local code = editor:GetText()
		local matches, m1, m2, m3, m4, tmp
		
		-- replace \n\n with \r\n\r\n
		code, m1 = string.gsub(code, "\n\n", '\r\n\r\n')
		--print("Matches LF LF", m1)
		
		-- replace orphan \n with \r\n
		tmp, m2 = string.gsub(' '..code, "([^\r])[\n]", '%1\r\n')
		if m2>0 then code = tmp.sub(tmp,2) end
		--print("Matches LF", m2)
		
		-- replace \r\r with \r\n\r\n
		code, m3 = string.gsub(code, "\r\r", '\r\n\r\n')
		--print("Matches CR CR", m3)
		
		-- replace orphan \r with \r\n
		tmp, m4 = string.gsub(code..' ', "[\r]([^\n])", '\r\n%1')
		if m4>0 then code = tmp.sub(tmp,1,-2) end
		--print("Matches CR", m4)
		
		if (m1+m2+m3+m4)>0 then editor:SetText(code) end

	elseif editor.EOLMode==1 then -- CR mode		
		-- remove \n
	  for m in editor:match('\n', 0) do
	    m:replace('\r')
	  end
	  
	else -- LF mode
		-- remove \r
	  for m in editor:match('\r', 0) do
	    m:replace('\n')
	  end
	end

end

----------------------------------------
--
----------------------------------------
local function extensionOf (s)
  return splitLast(s,'.')
end

----------------------------------------
--
----------------------------------------
--local function updateHandlers (fn)
--  output:ClearAll()
--  local ext = extensionOf(fn)
--  if ext=="ls" or ext=="lsw" then
--    functionList()
--  end
--end

----------------------------------------
--
----------------------------------------
local function fixCode(fn)
  local ext = extensionOf(fn)
  
  -- clean-up lingo scripts on open
  if ext=="ls" or ext=="lsw" then
  
  	-- remove trailing spaces
  	stripTrailSpaces()
  	
  	-- fix EOL according to current editor's setting/detection
  	fixEol()
  	
		-- fix indenting according to current editor's setting
		if not editor.UseTabs then
			tabsToSpaces()
		end
  
	end
end

----------------------------------------
-- Continues comment lines, until empty comment
----------------------------------------
local function autoContinueComment(line)

  -- block comment continue
  if line:find("^%s*%-%-%s*$") then --line is only '--', terminate comment
    editor:LineUp()
    editor:LineDelete()
  elseif line:find("^%s*%-%-") then --line is comment, auto continue it
    editor:AddText("-- ")
  end

  return false --allow other methods to respond to event
end

----------------------------------------
-- Show Director Help CHM (if word selected, navigate to corresponding index entry)
----------------------------------------
function showHelp ()
  local parms
  if props['CurrentWord']=="" then
    parms = '-DirHelp '..props['SciteDefaultHome']..'\\help\\D11.5_help.chm'
  else
    parms = '-DirHelp -#klink '..props['CurrentWord']..' '..props['SciteDefaultHome']..'\\help\\D11.5_help.chm'
  end
  winapi.shell_exec ('open', 'KeyHH.exe', parms, props['SciteDefaultHome']..'\\tools')
end

----------------------------------------
-- Creates clickable Lingo function list in Output pane
----------------------------------------
function functionList()

--  local str = ''
--  local code = "\r\n"..editor:GetText()
--  for res in string.gmatch(code, "[\r\n]on ([%a%d_]+)") do
--    str = str..res.."\n"
--  end
--  if (str) then
--    str = string.sub(str, 1, -2)
--  end
--  output:SetText(str)
  
  
	-- test: show as autocompletion list.

  local list = {}
  local code = "\r\n"..editor:GetText()
  for res in string.gmatch(code, "[\r\n]on ([%a%d_]+)") do
    table.insert(list, res)
  end
  table.sort(list)
  scite_UserListShow(list, 1, onHandlerSelected)
  
end

----------------------------------------
-- 
----------------------------------------
function onHandlerSelected(name)	
	local code = editor:GetText()
	local search = "on "..name

  if string.sub(code, 1, string.len(search))==search then
    pos = 0
  else
    pos = string.find(code, "[\n\r]"..search)
  end
  if pos then

    -- goto line
    local lineNum = editor:LineFromPosition(pos)
    editor:GotoLine(lineNum)

    -- select line
    posEnd = editor.LineEndPosition[lineNum]
    editor:SetSel(pos, posEnd) -- Select a range of text.
    editor.FirstVisibleLine = lineNum - 3
  end
	
end

----------------------------------------
-- 
----------------------------------------
function countQuotes (str)
	local cnt = 0
	for i in string.gfind(str, '"') do
		cnt = cnt + 1
	end
	return cnt
end

----------------------------------------
--
----------------------------------------
function stripComment (strLine)

--	local pos = string.find (strLine, "--", 1, true)
--	if pos==nil then return strLine end
--	return trim(string.sub(strLine, 1, pos-1))
	
-- better/ but slower version: checks if "--" is not 
-- inside string, by counting QUOTES before pos

	local pos, tmp
	local startPos = 1
	while true do
		pos = string.find(strLine, "--", startPos, true)
		if pos==nil then return strLine end
		
		tmp = string.sub(strLine, 1, pos-1)
		if countQuotes(tmp) % 2 == 0 then return trim(tmp) end
		startPos = pos + 2
	end

end

----------------------------------------
-- Re-indents/fromats current Lingo script
----------------------------------------
function reindent ()
  local code = editor:GetText()
  if code=="" then return end

  -- get editor's current EOL (0=CRLF, 1=CR, 2=LF)
  local eol
  if editor.EOLMode==0 then
    eol = '\r\n'
  elseif editor.EOLMode==1 then
    eol = '\r'
  else
    eol = '\n'
  end
  local eolLen = string.len(eol)

  -- get editor's current indent type
  local tabStr
  if editor.UseTabs then
    tabStr = '\t'
  else
    tabStr = string.rep(" ", editor.Indent)
  end
  local tabLen = string.len(tabStr)

  local res = ""
  local indent = ""
  local firstWord

  for line in getLines(code, eol) do

    -- trim line
    line = trim(line)

    -- get first word
    firstWord = string.match(line, "%w+")

    if firstWord=="on" or firstWord=="case" or firstWord=="repeat" then
      line = indent..line
      indent = tabStr..indent

    elseif firstWord=="end" then
      indent = string.sub(indent, tabLen+1)
      line = indent..line

    elseif firstWord=="if" then
      line = indent..line
      local tmp = stripComment(line)
       if string.sub(tmp, -4) == "then" then
        indent = tabStr..indent
      end
      
    elseif firstWord=="else" then
    	local tmp = stripComment(line)
    	if tmp=="else" then
    		line = string.sub(indent, tabLen+1)..line
    	else
    		line = indent..line
    	end
      
    elseif firstWord=="tell" then
      line = indent..line
      local tmp = stripComment(line)
      if not string.find(tmp, " to ") then
        indent = tabStr..indent
      end

    else
      line = indent..line

    end

    res = res..line..eol
  end

  if res~=code then editor:SetText(res) end
end

----------------------------------------
-- Runs current LSW script
----------------------------------------
function runLsw ()
  winapi.shell_exec('open', 'C:\\dev\\lsw\\lsw.exe', '"'..props['FilePath']..'"')
end

-- ADD MENU ITEMS --

scite_Command {
	'Director Help|showHelp|F1',
  'Handlers|functionList|*.ls;*.lsw|F2',
  'Re-indent Script|reindent|*.ls;*.lsw|Ctrl+Tab',
  --'Run LSW|runLsw|*.lsw|F5',
  --'Fix EOL|fixEol|'
}

-- EVENT HANDLERS --

scite_OnOpen(fixCode)
scite_OnBeforeSave(stripTrailSpaces)
scite_OnEditorLine(autoContinueComment)
	
--scite_OnOpen(updateHandlers)
--scite_OnSwitchFile(updateHandlers)
--scite_OnSave(updateHandlers)
--scite_OnDoubleClick(doubleClickHandler)

--scite_OnOutputLine(foo)
	
--
--scite_OnClose((function()
--	output:ClearAll()
--end))

--dbg(_VERSION)
--dbg_table(_G)
